#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os


class Configuration:
    def __init__(self, folder, name='config'):
        self.folder = folder
        self.file = os.path.join(folder, name + '.properties')
        self.props = self.load()

    def load(self):
        app_root_folder = os.path.dirname(__file__).replace("\\", "/")
        props = {}
        if not os.path.exists(self.file):
            return props
        with open(self.file, "rt") as f:
            for line in f:
                key_value = line.split('=')
                key = key_value[0].strip()
                if key != '':
                    value = '='.join(key_value[1:]).strip()
                    props[key] = value.replace("${APP_ROOT_FOLDER}", app_root_folder)
        return props

    def save(self, props=None):
        if props is None:
            props = {}
        with open(self.file, 'w+') as out:
            for key in props.keys():
                value = props[key]
                out.write(key + '=' + value + '\n')

    def get(self, key='key', default='default_value', save_if_no=True):
        value = self.props.get(key)
        if value is None:
            if save_if_no:
                return self.set(key, default)
            else:
                return default
        return value

    def set(self, key='key', value='value'):
        self.props[key] = value
        self.save(self.props)
        return value



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from datetime import datetime

from grutils import utils
from grutils import error
from grutils.utils import date_of

err = error.Error()
print(utils.int_value_of("101.12", err, 0))

print(date_of(datetime.date(datetime.now())))


#!/usr/bin/env python3
# -*- coding: utf-8 -*-


class MatchResult:
    def __init__(self, index=-1, data=None):
        self.index = index
        self.data = data


def first_match(match_function, iterable) -> MatchResult:
    i = 0
    for item in iterable:
        if match_function(item):
            return MatchResult(i, item)
        i += 1

    return MatchResult()


def all_matches(match_function, iterable) -> [MatchResult]:
    i = 0
    results = []
    for item in iterable:
        if match_function(item):
            results.append(MatchResult(i, item))
        i += 1

    return results



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from datetime import datetime, date
from typing import Dict, List, Optional, Tuple

from .error import Error


def strip_if_str(s):
    if type(s) == str:
        return s.strip()
    else:
        return s


def to_float(v):
    try:
        f = float(v)
        return True, f
    except Exception as e:
        return False, 'can not parse number from value: {}, error: {}'.format(v, e)


def to_int(v):
    is_float, result = to_float(v)
    if is_float:
        return True, int(round(result))

    return False, result


def int_value_of(v, err: Error, default_i=None):
    ok, result = to_int(v)
    if ok:
        return result
    else:
        err.append(result)
        return default_i


def parse_as_str_if_float_or_int(v):
    if type(v) == float:
        i = int(round(v, 0))
        return '{}'.format(i)
    elif type(v) == int:
        return '{}'.format(v)
    else:
        return v


def is_none_or_empty(s: str):
    return (type(s) == str and len(s) == 0) or (s is None)


def date_of(a: any):
    t = type(a)
    try:
        if t == str:
            if a.find('/') >= 0:
                items: List[str] = a.split('/')
                items_count = len(items)
                if items_count == 2:
                    month = int(items[0])
                    day = int(items[1])
                    today = datetime.date(datetime.now())
                    month_diff = today.month - month
                    year = today.year if (0 <= month_diff <= 8) else (today.year + 1)
                    return date.fromisoformat('{:04d}-{:02d}-{:02d}'.format(year, month, day))
                elif items_count == 3:
                    month = int(items[0])
                    day = int(items[1])
                    year = int(items[2])
                    return date.fromisoformat('{:04d}-{:02d}-{:02d}'.format(year, month, day))
            elif a.find('-') >= 0:
                items: List[str] = a.split('-')
                if len(items) == 3:
                    year = int(items[0])
                    month = int(items[1])
                    day = int(items[2])
                    return date.fromisoformat('{:04d}-{:02d}-{:02d}'.format(year, month, day))
            raise Exception('failed')
        elif t == datetime:
            return a.date()
        elif t == date:
            return a
        else:
            raise Exception('failed')
    except Exception as _:
        s = 'cannot parse date from ({})'.format(a)
        raise Exception(s)


def field_of(d: Dict[any, any], key: any):
    return None if d is None else (d[key] if key in d else None)


def float_02(f: float):
    return round(f * 100) / 100


def sort_and_merge_ints(ints: List[int]):
    results: List[Tuple[int, int]] = []
    if len(ints) == 0:
        return results

    sorted_ints = sorted(ints)

    start: Optional[int] = None
    curr: Optional[int] = None
    for i in sorted_ints:
        if start is None:
            start = i
            curr = i
            continue

        if i == curr + 1:
            curr = i
            continue

        results.append((start, curr))
        start = i
        curr = i

    if start is not None:
        results.append((start, curr))

    return results


def get_simple_desc_of_int_groups(groups: List[Tuple[int, int]],
                                  formatter: str = "{}",
                                  linker: str = "-",
                                  joiner: str = ","
                                  ):
    items = list(map(lambda group: (formatter + linker + formatter).format(group[0], group[1]) if group[0] != group[
        1] else formatter.format(group[0]), groups))
    return joiner.join(items)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


class Progress:
    def __init__(self):
        self.curr = 0
        self.max = 10000
        self.progress_bar = None

    def set_process_bar(self, process_bar):
        self.progress_bar = process_bar
        if self.progress_bar is not None:
            self.progress_bar["value"] = self.curr
            self.progress_bar["maximum"] = self.max

    def set(self, val: int):
        if val < 0:
            val = 0

        self.curr = val
        print("percent:", self.curr_percent())
        if self.progress_bar is not None:
            self.progress_bar["value"] = min(self.curr, self.max)
            self.progress_bar.update()

    def set_percent(self, percent: float):
        self.set(int(percent * self.max))

    def curr_percent(self):
        return float(self.curr) / float(self.max)

    def increase_percent(self, delta_percent: float):
        percent = self.curr_percent() + delta_percent
        self.set_percent(percent)

    def reset(self):
        self.set(0)

    def finish(self):
        self.set(self.max)

    def remaining(self):
        return self.max - self.curr

    def remaining_percent(self):
        return 1.0 - self.curr_percent()


shared_progress = Progress()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .error import Error
from .file import should_exists
from .form import Form
import chardet


def check_charset(file_path):
    with open(file_path, "rb") as f:
        data = f.read(4)
        charset = chardet.detect(data)['encoding']
    return charset


def load_form_from_csv_file(err: Error, file_path='', do_strip=True):
    if err.has_error():
        return None

    if not should_exists(err, file_path):
        return None

    data = Form(err)

    # 检查文件编码, 只对UTF-16做另外处理以额外支持GBK
    charset = check_charset(file_path)
    print('[Debug] file {} charset is {}'.format(file_path, charset))
    encoding = charset if charset == "UTF-16" else None
    splitter = '\t' if charset == "UTF-16" else ','
    with open(file_path, "rt", encoding=encoding) as f:
        line_no = 1
        for line in f:
            cells = split_line(line, do_strip, splitter)
            if len(cells) > 0:
                if data.title_row.is_inited():
                    data.append_data_row(line_no, cells, do_strip)
                else:
                    data.set_title_row(line_no, cells, do_strip)

            line_no += 1

    if err.has_error():
        return None
    return data


def split_line(line: str, do_strip=True, splitter=','):
    parts = line.split(splitter)
    res = []
    tmp_part = ''
    in_double_quotation_mark = False
    for part in parts:
        part_has_even_double_quotation_marks = part.count('"') % 2 == 0
        if in_double_quotation_mark:
            tmp_part = tmp_part + splitter + part
            if not part_has_even_double_quotation_marks:
                in_double_quotation_mark = False
                res.append(remove_double_quotation_marks(tmp_part))
        else:
            tmp_part = part
            if part_has_even_double_quotation_marks:
                res.append(remove_double_quotation_marks(tmp_part))
            else:
                in_double_quotation_mark = True

    if do_strip:
        return list(map(lambda x: x.strip(), res))


def remove_double_quotation_marks(part: str):
    part_len = len(part)
    if part_len > 1 and part[0] == '"' and part[part_len - 1] == '"':
        return part[1:part_len - 1]
    return part



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .error import Error


class Printer:
    def __init__(self, info_cb=None, err_cb=None):
        self.info_cb = info_cb
        self.err_cb = err_cb


global_printer = Printer()


def print_info(*args):
    s = ' '.join(map(lambda arg: '{}'.format(arg), args))
    if global_printer is None or global_printer.info_cb is None:
        print(s)
    else:
        global_printer.info_cb(s)


def print_err(err: Error, *args):
    s = '{}'.format(err.msg()) + ' '.join(map(lambda arg: '{}'.format(arg), args))
    if global_printer is None or global_printer.err_cb is None:
        print(s)
    else:
        global_printer.err_cb(s)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
from .error import Error


def should_exists(err: Error, file_path=''):
    if err.has_error():
        return False
    if not os.path.exists(file_path):
        err.append('file or folder "' + file_path + '" is not exists')
        return False
    return True


def new_path_with_postfix(file_path: str, postfix: str):
    folder = os.path.dirname(file_path)
    old_filename = os.path.basename(file_path)
    old_filename_items = os.path.splitext(old_filename)
    if len(old_filename_items) > 1:
        new_filename = old_filename_items[0] + postfix + old_filename_items[1]
    else:
        new_filename = old_filename + postfix
    return os.path.join(folder, new_filename)


def load_csv(err: Error, file_path=''):
    data = {
        'fields': [],
        'rows': []
    }

    if err.has_error():
        return data

    if not should_exists(err, file_path):
        return data

    with open(file_path, "rt") as f:
        for line in f:
            items = list(map(lambda x: x.strip(), line.split(',')))
            if len(items) == 0:
                continue

            if len(data.get('fields')) == 0:
                data['fields'] = items
            else:
                data['rows'].append(items)

    return data


def create_folder_if_not_exists(folder):
    if not os.path.exists(folder):
        os.mkdir(folder)



from setuptools import setup, find_packages
from grutils import __author__, __version__, __email__

# read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

requirementPath = this_directory + '/requirements.txt'
install_requires = []
with open(requirementPath) as f:
    install_requires = f.read().splitlines()

setup(
    name='grutils',
    version=__version__,
    author=__author__,
    author_email=__email__,
    license='Apache2.0',
    description='reusable grutils',
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        'Topic :: Software Development :: Libraries',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],
    python_requires='>=3.6',
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    install_requires=install_requires,
    project_urls={
        'Source': 'https://github.com/lldld/gr.py3.utils'
    },
)
